{% extends 'layouts/default.html' %}
{% from 'macros.html' import build_ip_tbody, build_rtbh_tbody, build_rules_thead %}

{% block title %}Flowspec{% endblock %}
{% block content %}

    {% include 'pages/submenu_dashboard.html' %}

    <div class="row">
        <table class="table table-hover ip-table">
            {{ build_rules_thead(rules_columns, rtype, rstate, sort_key, sort_order) }}
            {% if rtype_int == 1 %}
                {{ build_rtbh_tbody(rules, today, rtype_int) }}
            {% else %}
                {{ build_ip_tbody(rules, today, rtype_int) }}
            {% endif %}
        </table>
    </div>

{% endblock %}