{% extends 'layouts/default.html' %}

{% block title %}Flowspec{% endblock %}
{% block content %}
    <h2>WHOIS for {{ ip_address }}</h2>

    <div class="row">
        <pre>{{ result }}</pre>        
    </div>

{% endblock %}