body {
    font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
    font-size: 14px;
    line-height: 1.42857143;
    color: #333;
    background-color: #fff;
    padding-top: 70px;
    padding-bottom: 30px;
}

label {
  display: inline-block;
  max-width: 100%;
  margin-bottom: 5px;
  font-weight: 700;
}

.form-group {
  margin-bottom: 15px;
}

.h1, .h2, .h3, h1, h2, h3 {
  margin-top: 20px;
  margin-bottom: 10px;
}

.h1, .h2, .h3, .h4, .h5, .h6, h1, h2, h3, h4, h5, h6 {
  font-family: inherit;
  font-weight: 500;
  line-height: 1.1;
  color: inherit;
}

.h2, h2 {
  font-size: 30px;
}

#dashboard-search {
    margin-top: 10px;
}

.theme-dropdown .dropdown-menu {
    position: static;
    display: block;
    margin-bottom: 20px;
}

.theme-showcase > p > .btn {
    margin: 5px 0;
}

.theme-showcase .navbar .container {
    width: auto;
}


tr.bottom_row > td {
    border-bottom: solid black 1px;
    padding-bottom: 1em;
}

tr.bottom_row > th {
    border-bottom: solid black 1px;
    padding-bottom: 1em;
}


td.bottom_row {
    border-bottom: solid black 1px;
    padding-bottom: 1em;
}


.ip-table thead {
    background-color: #adadad;
    cursor: pointer;
}

#dashboard-nav .nav > li > a {
    padding-top: 5px;
    padding-bottom: 5px;
    margin-top: 10px;
    font-size: 130%;
}

#dashboard-nav a.active {
    background-color: #1b67ad;
    color: white;
}

#dashboard-nav h1 {
    font-size: 130%;
    font-weight: bold;
    margin-top: 10px;
}


.sort-direction {
    display: inline-block;
    height: 0;
    width: 0;
    border-left: 5px solid transparent;
    border-right: 5px solid transparent;
}

.asc {
    border-bottom: 5px solid black;
}

.desc {
    border-top: 5px solid black;
}

.table-sorter a {
    color: #0a0a0a;
    text-decoration: none;
}

.table-sorter a:visited {
    color: #0a0a0a;
}

/* tasks */

.tasks {
  list-style: none;
  margin: 0;
  padding: 0;
}

.task {
  justify-content: space-between;
}

.task:last-child {
  border-bottom: none;
}

/* context menu */

.context-menu {
  display: none;
  position: absolute;
  z-index: 10;
  padding: 12px 0;
  width: 240px;
  background-color: #fff;
  border: solid 1px #dfdfdf;
  box-shadow: 1px 1px 2px #cfcfcf;
}

.context-menu--active {
  display: block;
}

.context-menu__items {
  list-style: none;
  margin: 0;
  padding: 0;
}

.context-menu__item {
  display: block;
  margin-bottom: 4px;
}

.context-menu__item:last-child {
  margin-bottom: 0;
}

.context-menu__link {
  display: block;
  padding: 4px 12px;
  color: #0066aa;
  text-decoration: none;
}

.context-menu__link:hover {
  color: #fff;
  background-color: #0066aa;
}

tbody .bi {
  -webkit-text-stroke: 1px;
  font-size: 105%;
  color: white;
}

.alert {
  border-radius: 10px;
}
