
{% block submenu_dashboard %}

  <h1>{{ rstate|capitalize }} {{ table_title }}</h1>

    <div class="row">
             <div class="container" id="dashboard-nav">
                 <div class="col-md-6">
                     <ul class="nav nav-pills">
                         {% for key, item in dashboard.items() %}
                            <li class="nav-item">
                                <a class="nav-link {{ css_classes[key] }}" href="{{ url_for('dashboard.index', rtype=key, rstate=rstate) }}" >
                                {{ item.name }} 
                                {% if count_match %}
                                    ({{ count_match[key] }})
                                {% endif %}
                                </a>
                            </li>
                        {% endfor %} 
                     </ul>
                 </div>
                 <div class="col-md-6">
                     <ul class="nav nav-pills pull-right">
                         <li class="nav-item"> 
                             {% include 'pages/dashboard_search_form.html' %}
                         </li>
                         <li class="nav-item">
                             <a class="nav-link {{ css_classes['active'] }}" href="{{ url_for('dashboard.index', rtype=rtype, rstate='active') }}" >Active</a>
                         </li>
                         <li class="nav-item">
                             <a class="nav-link {{ css_classes['expired'] }}" href="{{ url_for('dashboard.index', rtype=rtype, rstate='expired') }}" >Expired</a>
                         </li>
                         <li class="nav-item">
                             <a class="nav-link {{ css_classes['all'] }}" href="{{ url_for('dashboard.index', rtype=rtype, rstate='all') }}" >All</a>
                         </li>
                     </ul>
                 </div>
    </div>
    <hr />

{% endblock %}