{% extends 'layouts/default.html' %}
{% from 'forms/macros.html' import render_form %}

{% block title %}
{{ title }}
{% endblock %}
{% block content %}
    <div class="simple-form">
        {{ render_form(form, action_url=action_url, action_text='Save') }}
    </div>
{% endblock content %}