{% extends 'layouts/default.html' %}
{% block title %}AS Paths{% endblock %}
{% block content %}
    <table class="table table-hover">
    <tr>
    <th>Prefix</th>
    <th>AS-path</th>
    <th>Action</th>
    </tr>
    {% for pth in paths %}
        <tr>
        <td>{{ pth.prefix }}</td>
        <td>
            {{ pth.as_path }}
        </td>
        <td>
        <a class="btn btn-info btn-sm" href="{{ url_for('admin.edit_as_path', path_id=pth.id) }}" role="button">
            <i class="bi bi-pen"></i>
        </a>
        <a class="btn btn-danger btn-sm" href="{{ url_for('admin.delete_as_path', path_id=pth.id) }}" role="button">
            <i class="bi bi-x-lg"></i>
        </a>
        </td>
        </tr>
    {% endfor %}
    </table>
{% endblock %}