{% extends 'layouts/default.html' %}
{% block title %}Flowspec Actions{% endblock %}
{% block content %}
    <table class="table table-hover">
    <tr>
    <th>Id</th>
    <th>Name</th>
    <th>Command</th>
    <th>Description</th>
    <th>Minimum level</th>
    <th>action</th>
    </tr>
    {% for action in actions %}
        <tr>
        <td>{{ action.id }}</td>
        <td>{{ action.name }}</td>
        <td>{{ action.command }}</td>
        <td>
            {{ action.description }}
        </td>
        <td>
            {{ action.role }}
        </td>
        <td>
        <a class="btn btn-info btn-sm" href="{{ url_for('admin.edit_action', action_id=action.id) }}" role="button">
            <i class="bi bi-pen"></i>
        </a>
        <a class="btn btn-danger btn-sm" href="{{ url_for('admin.delete_action', action_id=action.id) }}" role="button">
            <i class="bi bi-x-lg"></i>
        </a>
        </td>
        </tr>
    {% endfor %}
    </table>
{% endblock %}